package ru.myitschool.work.data

import android.util.Log
import ru.myitschool.work.data.dto.UserDto
import ru.myitschool.work.data.local.CredentialsLocalDataSource
import ru.myitschool.work.data.local.UserLocalDataSource
import ru.myitschool.work.domain.entities.UserEntity
import ru.myitschool.work.domain.login.LoginRepository
import ru.myitschool.work.data.network.UserNetworkDataSource
import ru.myitschool.work.domain.user.UserRepository

class UserRepositoryImpl(
    private val credentialsLocalDataSource: CredentialsLocalDataSource,
    private val userLocalDataSource: UserLocalDataSource,
    private val networkDataSource: UserNetworkDataSource
) : LoginRepository, UserRepository {

    override suspend fun login(login: String, password: String): Result<Unit> {

        return runCatching {
            networkDataSource.login(credentialsLocalDataSource.updateToken(login, password))
                .onSuccess { dto ->
                    map(dto).onSuccess { userLocalDataSource.cacheData(it) }
                }
        }
    }

    override suspend fun authorize(token: String): Result<Unit> {
        return networkDataSource.login(token).fold(
            onSuccess = { Result.success(Unit) },
            onFailure = { error -> Result.failure(error) }
        )
    }

    override suspend fun logout() {
        credentialsLocalDataSource.clear()
        userLocalDataSource.clear()
    }

    override suspend fun isUserExist(login: String): Result<Boolean> {
        return networkDataSource.isUserExist(login)
    }

    override suspend fun getCurrentUser(): UserEntity {
        return userLocalDataSource.getUser()!!
    }

    private fun map(userDto: UserDto): Result<UserEntity> {
        return runCatching {
            UserEntity(
                id = userDto.id ?: error("Null user id"),
                name = userDto.name ?: error("Null user name"),
                lastVisit = userDto.lastVisit ?: error("Null user lastVisit"),
                photoUrl = userDto.photoUrl ?: error("Null user photoUrl"),
                position = userDto.position ?: error("Null user position")
            )
        }
    }
}