package ru.myitschool.work.ui.login

import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.core.widget.doAfterTextChanged
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import ru.myitschool.work.R
import ru.myitschool.work.databinding.FragmentLoginBinding
import ru.myitschool.work.utils.collectWithLifecycle
import ru.myitschool.work.utils.visibleOrGone

class LoginFragment : Fragment(R.layout.fragment_login) {

    private var _binding: FragmentLoginBinding? = null
    private val binding: FragmentLoginBinding get() = _binding!!

    private val viewModel by viewModels<LoginViewModel> { LoginViewModel.Factory }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        _binding = FragmentLoginBinding.bind(view)

        binding.username.doAfterTextChanged { login ->
            viewModel.changeLogin(login = login.toString())
        }

        viewModel.state.collectWithLifecycle(this) { state ->
            binding.username.isEnabled = state !is LoginViewModel.State.Loading
            binding.password.isEnabled = state !is LoginViewModel.State.Loading
            binding.error.visibleOrGone(state is LoginViewModel.State.Error)

            when (state) {
                is LoginViewModel.State.Error -> binding.error.text = state.errorMessage
                is LoginViewModel.State.Loading -> Unit
                is LoginViewModel.State.Waiting -> Unit
                is LoginViewModel.State.LoginCheckCompleted -> binding.login.isEnabled =
                    state.isCompleted
            }
        }

        viewModel.action.collectWithLifecycle(this) { action ->
            when (action) {
                is LoginViewModel.Action.OpenApp ->
                    findNavController().navigate(R.id.action_loginFragment_to_userFragment)

                is LoginViewModel.Action.GoToLogin -> Unit
            }
        }

        binding.login.setOnClickListener {
            viewModel.onProcessClick(
                login = binding.username.text.toString(),
                password = binding.password.text.toString(),
            )
        }
    }

    override fun onDestroy() {
        _binding = null
        super.onDestroy()
    }
}